/**
 * Copyright 1998-2007, CHISEL Group, University of Victoria, Victoria, BC, Canada.
 * All rights reserved.
 */
package ca.uvic.csr.shrimp.gui.HierarchicalView;

import java.util.Iterator;
import java.util.Vector;

import ca.uvic.csr.shrimp.SelectorBeanConstants;
import ca.uvic.csr.shrimp.DisplayBean.DisplayBean;
import ca.uvic.csr.shrimp.DisplayBean.ShrimpNode;
import ca.uvic.csr.shrimp.DisplayBean.PFlatDisplayBean.PFlatDisplayBean;
import ca.uvic.csr.shrimp.FilterBean.FilterBean;
import ca.uvic.csr.shrimp.SelectorBean.SelectorBean;
import ca.uvic.csr.shrimp.ShrimpApplication.BeanNotFoundException;
import ca.uvic.csr.shrimp.ShrimpProject.ShrimpProject;
import ca.uvic.csr.shrimp.ShrimpTool.ShrimpTool;
import ca.uvic.csr.shrimp.adapter.HvDisplayModeChangeAdapter;
import ca.uvic.csr.shrimp.adapter.HvFilterChangedAdapter;
import ca.uvic.csr.shrimp.adapter.HvFilterChangedAdapterForHiding;
import ca.uvic.csr.shrimp.adapter.HvNodeDoubleClickAdapter;
import ca.uvic.csr.shrimp.adapter.HvSelectedNodesChangeAdapter;
import ca.uvic.csr.shrimp.gui.ShrimpView.ShrimpView;

/**
 * Provides some utilities for converting objects between the HierarchicalView and the ShrimpView.
 * 
 * @author Rob Lintern
 */
public class HvSvBridge {

	private static ShrimpView sv;
	private static HierarchicalView hv;

	// Hierarchical View Adapters
	private static HvDisplayModeChangeAdapter hvdmca; // listens for changes in the hier view's display mode
	private static HvFilterChangedAdapter hvfca; // listens for changes in a filter bean, to update hier views "filtered" mode
	private static HvFilterChangedAdapterForHiding hvfcafh; // listens for changes in a filter bean, if hier view is in "hide filtered" modes
	// private HvHideFilteredModeChangeAdapter hvhfmca; //listens for changes to the hier view's "hide filtered" mode.
	private static HvNodeDoubleClickAdapter hvndca; // listens for double clicking of nodes in the hierarchical view
	private static HvSelectedNodesChangeAdapter hvsnca; // accentuates selected nodes in hierarhical view

	/**
	 * Converts a vector of nodes from the hierView into a vector of nodes in the shrimpView that represent the same artifacts.
	 */
	public static Vector hvNodesToSvNodes(HierarchicalView hierarchicalView, ShrimpView shrimpView, Vector hvNodes, boolean createIfNoneFound) {
		Vector svNodes = new Vector();
		try {
			DisplayBean svDisplayBean = (DisplayBean) shrimpView.getBean(ShrimpTool.DISPLAY_BEAN);
			for (Iterator iter = hvNodes.iterator(); iter.hasNext();) {
				ShrimpNode hvNode = (ShrimpNode) iter.next();
				svNodes.addAll(svDisplayBean.getDataDisplayBridge().getShrimpNodes(hvNode.getArtifact(), createIfNoneFound));
			}
		} catch (BeanNotFoundException e) {
			e.printStackTrace();
		}
		return svNodes;
	}

	/**
	 * Converts a vector of nodes from the ShrimpView into a vector of nodes in the hierView that represent the same artifacts.
	 */
	public static Vector svNodesToHvNodes(HierarchicalView hierarchicalView, ShrimpView shrimpView, Vector svNodes, boolean createIfNoneFound) {
		Vector hvNodes = new Vector();
		try {
			PFlatDisplayBean flatDisplayBean = (PFlatDisplayBean) hierarchicalView.getBean(ShrimpTool.DISPLAY_BEAN);
			for (Iterator iter = svNodes.iterator(); iter.hasNext();) {
				ShrimpNode svNode = (ShrimpNode) iter.next();
				hvNodes.addAll(flatDisplayBean.getDataDisplayBridge().getShrimpNodes(svNode.getArtifact(), createIfNoneFound));
			}
		} catch (BeanNotFoundException e) {
			e.printStackTrace();
		}
		return hvNodes;
	}

	// create some links between the two views
	public static void joinShrimpViewHierarchicalView(ShrimpView shrimpView, HierarchicalView hierarchicalView, ShrimpProject project) {
		if (sv != null && hv != null && sv.equals(shrimpView) && hv.equals(hierarchicalView)) {
			return; // these views are already linked
		}
		sv = shrimpView;
		hv = hierarchicalView;
		// System.err.println("joining HV and SV: ");

		// create adapters
		hvdmca = new HvDisplayModeChangeAdapter(sv, hv, project);
		hvfca = new HvFilterChangedAdapter(sv, hv);
		hvfcafh = new HvFilterChangedAdapterForHiding(hv);
		//hvhfmca = new HvHideFilteredModeChangeAdapter(project, sv, hv);
		hvndca = new HvNodeDoubleClickAdapter(sv, hv);
		hvsnca = new HvSelectedNodesChangeAdapter(sv, hv);

		// add adapters to appropriate places
		hierarchicalView.addDisplayModeChangeListener(hvdmca);
		hierarchicalView.addNodeDoubleClickListener(hvndca);
		// hierarchicalView.addHideFilteredModeChangeListener(hvhfmca);

		try {
			FilterBean svFilterBean = (FilterBean) shrimpView.getBean(ShrimpTool.DISPLAY_FILTER_BEAN);
			SelectorBean selectorBean = (SelectorBean) shrimpView.getBean(ShrimpTool.SELECTOR_BEAN);
			hierarchicalView.registerExternalFilterBean(svFilterBean);
			svFilterBean.addFilterChangedListener(hvfca);
			svFilterBean.addFilterChangedListener(hvfcafh);
			selectorBean.addPropertyChangeListener(SelectorBeanConstants.SELECTED_NODES, hvsnca);
		} catch (BeanNotFoundException e) {
			e.printStackTrace();
		}
	}

	public static void separateShrimpViewHierarchicalView(ShrimpView shrimpView, HierarchicalView hierarchicalView) {
		if ((shrimpView != null) && (hierarchicalView != null)) {
			// System.err.println("seperating HV and SV: ");
			hierarchicalView.removeDisplayModeChangeListener(hvdmca);
			// hierarchicalView.removeHideFilteredModeChangeListener(hvhfmca);
			hierarchicalView.removeNodeDoubleClickListener(hvndca);
	
			try {
				FilterBean svFilterBean = (FilterBean) shrimpView.getBean(ShrimpTool.DISPLAY_FILTER_BEAN);
				svFilterBean.removeFilterChangedListener(hvfca);
				svFilterBean.removeFilterChangedListener(hvfcafh);
				hierarchicalView.unregisterExternalFilterBean(svFilterBean);
			} catch (BeanNotFoundException e) {
				e.printStackTrace();
			}
	
			try {
				SelectorBean selectorBean = (SelectorBean) shrimpView.getBean(ShrimpTool.SELECTOR_BEAN);
				selectorBean.removePropertyChangeListener(SelectorBeanConstants.SELECTED_NODES, hvsnca);
			} catch (BeanNotFoundException e1) {
				e1.printStackTrace();
			}
		}

		hvdmca = null;
		hvfca = null;
		hvfcafh = null;
		// hvhfmca = null;
		hvndca = null;
		hvsnca = null;

		sv = null;
		hv = null;
	}

}
